@@ -1,6 +1,6 @@
 package Devel::Confess;
 BEGIN {
-  my $can_use_informative_names = $] >= 5.8;
+  my $can_use_informative_names = $] >= 5.008;
   # detect -d:Confess.  disable debugger features for now.  we'll
   # enable them when we need them.
   if (!defined &DB::DB && $^P & 0x02) {
@@ -9,20 +9,24 @@ BEGIN {
   }
   *_CAN_USE_INFORMATIVE_NAMES
     = $can_use_informative_names ? sub () { 1 } : sub () { 0 };
-  *_BAD_CLONE_DESTROY
-    = ($] => 5.008009 && $] <= 5.010000) ? sub () { 1 } : sub () { 0 };
+  *_BROKEN_CLONED_DESTROY_REBLESS
+    = ($] >= 5.008009 && $] < 5.010000) ? sub () { 1 } : sub () { 0 };
+  *_BROKEN_CLONED_GLOB_UNDEF
+    = ($] > 5.008009 && $] <= 5.010000) ? sub () { 1 } : sub () { 0 };
+  *_BROKEN_SIG_DELETE
+    = ($] < 5.008008) ? sub () { 1 } : sub () { 0 };
 }
 
 use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '0.007009';
+our $VERSION = '0.007010';
 $VERSION = eval $VERSION;
 
 use Carp ();
 use Symbol ();
-use Devel::Confess::_Util qw(blessed refaddr weaken longmess);
+use Devel::Confess::_Util qw(blessed refaddr weaken longmess _str_val);
 BEGIN { *_can = \&UNIVERSAL::can; }
 
 $Carp::Internal{+__PACKAGE__}++;
@@ -112,15 +116,27 @@ sub import {
 }
 
 sub unimport {
-  return
-    unless keys %OLD_SIG;
-  for (qw(__DIE__ __WARN__)) {
-    my $sig = delete $OLD_SIG{$_};
-    if (defined $sig) {
-      $SIG{$_} = $sig;
+  for my $sig (
+    [ __DIE__ => \&_die ],
+    [ __WARN__ => \&_warn ],
+  ) {
+    my ($name, $sub) = @$sig;
+    my $now = $SIG{$name} or next;
+    my $old = $OLD_SIG{$name};
+    if ($now ne $sub && $old) {
+      local $SIG{__WARN__};
+      warn "Can't restore $name handler!\n";
+      delete $SIG{$sig};
+    }
+    elsif ($old) {
+      $SIG{$name} = $old;
+      delete $OLD_SIG{$name};
     }
     else {
-      delete $SIG{$_};
+      no warnings 'uninitialized'; # bogus warnings on perl < 5.8.8
+      undef $SIG{$name}
+        if _BROKEN_SIG_DELETE;
+      delete $SIG{$name};
     }
   }
 }
@@ -190,8 +206,8 @@ sub _ref_formatter {
 
 sub _stack_trace {
   no warnings 'once';
-  local $Carp::RefArgFormatter = \&_ref_formatter
-    if $OPTIONS{dump};
+  local $Carp::RefArgFormatter
+    = $OPTIONS{dump} ? \&_ref_formatter : \&_str_val;
   my $message = &longmess;
   $message =~ s/\.?$/./m;
   if ($OPTIONS{source}) {
@@ -205,26 +221,28 @@ our $PACK_SUFFIX = 'A000';
 our %EXCEPTIONS;
 our %PACKAGES;
 our %MESSAGES;
-
-sub _update_ex_refs {
-  for my $old_id ( keys %EXCEPTIONS ) {
-    my $package = delete $PACKAGES{$old_id};
-    my $message = delete $MESSAGES{$old_id};
-    my $ex = delete $EXCEPTIONS{$old_id};
-    next
-      unless defined $ex;
-    my $id = refaddr($ex);
-    weaken($EXCEPTIONS{$id} = $ex);
-    $PACKAGES{$id} = $package;
-    $MESSAGES{$id} = $message;
-  }
-}
+our %CLONED;
 
 sub CLONE {
-  _update_ex_refs;
-  if (_BAD_CLONE_DESTROY) {
-    our %CLONED;
-    @CLONED{keys %EXCEPTIONS} = ();
+  my %id_map = map { my $ex = $EXCEPTIONS{$_}; $ex ? ($_ => refaddr($ex)) : () } keys %EXCEPTIONS;
+  %EXCEPTIONS = map {; $id_map{$_} => $EXCEPTIONS{$_}} keys %id_map;
+  %PACKAGES = map {; $id_map{$_} => $PACKAGES{$_}} keys %id_map;
+  %MESSAGES = map {; $id_map{$_} => $MESSAGES{$_}} keys %id_map;
+  %CLONED = map {; $_ => 1 } values %id_map
+    if _BROKEN_CLONED_DESTROY_REBLESS || _BROKEN_CLONED_GLOB_UNDEF;
+  weaken($_)
+    for values %EXCEPTIONS;
+}
+
+sub _update_ex_refs {
+  for my $id ( keys %EXCEPTIONS ) {
+    next
+      if $EXCEPTIONS{$id};
+    delete $EXCEPTIONS{$id};
+    delete $PACKAGES{$id};
+    delete $MESSAGES{$id};
+    delete $CLONED{$id}
+      if _BROKEN_CLONED_DESTROY_REBLESS || _BROKEN_CLONED_GLOB_UNDEF;
   }
 }
 
@@ -367,15 +385,26 @@ sub _ex_as_strings {
 
     my $newclass = ref $ex;
 
-    if (Devel::Confess::_BAD_CLONE_DESTROY && exists $Devel::Confess::CLONED{$id}) {
+    my $cloned;
+    # delete_package is more complete, but can explode on some perls
+    if (Devel::Confess::_BROKEN_CLONED_GLOB_UNDEF && delete $Devel::Confess::CLONED{$id}) {
+      $cloned = 1;
+      no strict 'refs';
+      @{"${newclass}::ISA"} = ();
+      my $stash = \%{"${newclass}::"};
+      delete @{$stash}{keys %$stash};
+    }
+    else {
+      Symbol::delete_package($newclass);
+    }
+
+    if (Devel::Confess::_BROKEN_CLONED_DESTROY_REBLESS && $cloned || delete $Devel::Confess::CLONED{$id}) {
       my $destroy = $class->can('DESTROY') || return;
       goto $destroy;
     }
 
     bless $ex, $class;
 
-    Symbol::delete_package($newclass);
-
     # after reblessing, perl will re-dispatch to the class's own DESTROY.
     ();
   }
